import javax.swing.*;

public class DFT_Example extends JFrame{
	JTextArea textArea = new JTextArea(40, 40);

	private static String THETA = "\u0398"; // Unicode theta

	public DFT_Example(){
		setTitle("DFT of a ramp function");
		getContentPane().add(new JScrollPane(textArea));
		setSize(400, 400);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public void go(){
		Complex ramp[];
		Complex transform[];
		Complex inverse[];
		
		// display input
		
		textArea.append("input ramp\n\n");
		ramp = new Complex[8];
		for (int i = 0; i < 8; i++){
			ramp[i] = new Complex(1/8f * i, 0);
			textArea.append("ramp[" + i + "] = " + formatComplex(ramp[i]) + "\t");
			textArea.append("r = " + formatReal(ramp[i].getMagnitude()) + "\t");
			textArea.append(THETA + " = " + formatReal(ramp[i].getAngle()) + "\n");
		}
		textArea.append("\n");
		
		// display transform
		
		transform = DFT.computeTransform(ramp);
		textArea.append("transform\n\n");
		for (int i = 0; i < 8; i++){
			textArea.append("transform[" + i + "] = " + formatComplex(transform[i]) + "\t");
			textArea.append("r = " + formatReal(transform[i].getMagnitude()) + "\t");
			textArea.append(THETA + " = " + formatReal(transform[i].getAngle()) + "\n");
		}
		textArea.append("\n");

		// display inverse
		
		inverse = DFT.computeInverse(transform);
		textArea.append("inverse\n\n");
		for (int i = 0; i < 8; i++){
			textArea.append("inverse[" + i + "] = " + formatComplex(inverse[i]) + "\t");
			textArea.append("r = " + formatReal(inverse[i].getMagnitude()) + "\t");
			textArea.append(THETA + " = " + formatReal(inverse[i].getAngle()) + "\n");
		}
	}
	
	public String formatReal(float x){
		x = Math.round(x*1000.0)/1000f;
		return new Float(x).toString();
	}	
	
	public String formatComplex(Complex x){
		float r = x.getReal();
		float i = x.getImag();
		
		// round for display
		r = Math.round(r*1000.0)/1000f;
		i = Math.round(i*1000.0)/1000f;
	
		// convert to string
		
		return new Float(r).toString() +
			(i>=0?" + ":" - ") +
			Math.abs(i) +
			"j";
	}

	public static void main(String[] args){
		DFT_Example x = new DFT_Example();
		x.show();
		x.go();
	}
}