import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.*;

class SquarePulseTransform extends JFrame{
	JTextField indexField = new JTextField(10);
	JTextField dutyCycleField = new JTextField(10);
	JTextField realOutputField = new JTextField(10);
	JTextField imagOutputField = new JTextField(10);
	JButton computeButton = new JButton("Compute");
	JTextArea tableArea = new JTextArea(40, 60);

	public SquarePulseTransform(){
		super("Square Pulse Transform Coefficients");
		
		setSize(450, 300);
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		realOutputField.setEditable(false);
		imagOutputField.setEditable(false);
		tableArea.setEditable(false);

		Container c = getContentPane();
		
		c.setLayout(new FlowLayout());
		c.add(new JLabel("coeff index k"));
		c.add(indexField);
		c.add(new JLabel("duty cycle"));
		c.add(dutyCycleField);
		c.add(new JLabel("coeff real part"));
		c.add(realOutputField);
		c.add(new JLabel("coeff imag part"));
		c.add(imagOutputField);
		c.add(computeButton);
		c.add(tableArea);
		
		KeyListener localKeyListener = new KeyListener(){
			public void keyPressed(KeyEvent e){
			}		
			public void keyReleased(KeyEvent e){
			}		
			public void keyTyped(KeyEvent e){
				argumentChanged();
			}		
		};
		indexField.addKeyListener(localKeyListener);
		dutyCycleField.addKeyListener(localKeyListener);
		
		ActionListener localButtonListener = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				computeButtonPressed();
			}
		};
		computeButton.addActionListener(localButtonListener);
		
	}

	private double computeCoeffReal(int k, double dutyCycle){
		double two_pi_k = 2.0 * Math.PI * k;
		
		if (k == 0)
			return dutyCycle;
		else
			return (1/two_pi_k) * Math.sin(two_pi_k * dutyCycle);
	}

	private double computeCoeffImag(int k, double dutyCycle){
		double two_pi_k = 2.0 * Math.PI * k;
	
		if (k == 0)
			return 0;
		else
			return (1/two_pi_k) * (Math.cos(two_pi_k * dutyCycle) - 1.0);
	}
	
	// round a number to 9 decimal places for display
	private String formatNumber(double x){
		DecimalFormat df = new DecimalFormat();
		df.setMinimumFractionDigits(0);
		df.setMaximumFractionDigits(9);
		
		return df.format(x + 5e-11);
	}
	
	private void argumentChanged(){
		realOutputField.setText("");
		imagOutputField.setText("");
		tableArea.setText("");
	}

	private void computeButtonPressed(){
		double dutyCycle;
		int k;
		
		try{
			dutyCycle = Double.parseDouble(dutyCycleField.getText());
			k = Integer.parseInt(indexField.getText());
			
			if (dutyCycle <0 || dutyCycle > 1){
				throw new Exception();
			}
			
		}catch (Exception e){
			JOptionPane.showMessageDialog(this, "bad input");
			return;
		}
		
		realOutputField.setText(
				formatNumber(computeCoeffReal(k, dutyCycle))
			);
		imagOutputField.setText(
				formatNumber(computeCoeffImag(k, dutyCycle))
			);
			
		tableArea.append("alpha\tk\tRe(c(k))\tIm(c(k))\n");
		for (k=-4; k<=4; k++){
			String line;
			
			line = dutyCycle + "\t" + 
			       k + "\t" + 
			       formatNumber(computeCoeffReal(k, dutyCycle)) + "\t" + 
			       formatNumber(computeCoeffImag(k, dutyCycle)) + "\n";

			tableArea.append(line);		
		}		
	}
	
	public static void main(String args[]){
		SquarePulseTransform transformFrame = new SquarePulseTransform();
		transformFrame.show();
	}
}