/************************************************************
 * Glenn Josefiak                                           *
 * SW511                                                    *
 * Fairfield University                                     *
 * Fall 2002                                                *
 ************************************************************/

package dsp;

/************************************************************/
/**
 *	FIR filter with floating-point coefficents and 16-bit
 *  linear fixed-point PCM sample values.  This may need
 *  to be extended.
 */
public class FIRFilter{
	private float coeffs[];
	private short delay[];
	private	short data[];
	private int oldBufferLength;

	/********************************************************/
	/**
	 *	Creates a new FIR filter with the specified coefficients.
	 */
	public FIRFilter(float coeffs[]){
		this.coeffs = coeffs;
		delay = new short[coeffs.length];
		for (int i = 0; i< delay.length; i++){
			delay[i] = 0;
		}
	}
	
	/********************************************************/
	/**
	 *	Replaces the array of samples with a filtered version
	 *  of itself.
	 */
	public void compute(short buffer[]){
		int i, j;
		
		// append new buffer to delay line
		if (data == null || buffer.length != oldBufferLength){
			data = new short[coeffs.length + buffer.length];
			oldBufferLength = buffer.length;
		}
		for (i=0; i<coeffs.length; i++){
			data[i] = delay[i];
		}
		for (i=0; i<buffer.length; i++){
			data[coeffs.length+i] = buffer[i];
		}
		
		// compute filtered signal sample by sample and store result
		// back in the original buffer.
		for (i = 0; i< buffer.length; i++){
			float sample = 0;
			for (j = 0; j<coeffs.length; j++){
				sample += data[i+j]*coeffs[j];
			}
			buffer[i] = (short)Math.min(sample, (float)Short.MAX_VALUE);			
		}
		
		// store the latest delayed samples for next time
		for(i=0; i<delay.length; i++){
			delay[i] = data[i+buffer.length];
		}	
	}	
}