/************************************************************
 * Glenn Josefiak                                           *
 * SW511                                                    *
 * Fairfield University                                     *
 * Fall 2002                                                *
 ************************************************************/

package gui;

import java.awt.*;
import javax.swing.*;

/************************************************************/
/**
 * Displays a logarithmic bar graph of signal energy.  
 * The signal energy values are expected to be derived
 * as mean square values of an array of signed 16-bit
 * audio samples.
 */
public class NoteDisplay extends JPanel{
	private static final double LOG10 = Math.log(10);
	
	// Signal dynamic range displayed on the graph (in dB)
	private static final float DYNAMIC_RANGE_dB = 50;

	// This is the mean square value of a square wave whose
	// peak value is 32767, the max value of 'short'.
	private static final float MAX_MEAN_SQUARE = 32767*32767;

	// local copy of the power value array
	private	float power[];
	
	/********************************************************/
	/**
	 * Construct a new NoteDisplay panel.
	 */
	public NoteDisplay(){
		setBackground(Color.black);
		setPreferredSize(new Dimension(150,150));
		power = new float[0];
	}
	
	/********************************************************/
	/**
	 * Tell the NoteDisplay to graph the supplied note amplitudes.
	 */
	public void setData(float[] power){
		this.power = power;
		repaint();
	}

	/********************************************************/
	/**
	 * Override JComponent.paint().
	 */
    public void paint(Graphics g) {
    	// Erase panel area
		g.setColor(Color.black);
		g.fillRect(0, 0, getWidth(), getHeight());
		
		// Draw the bars
		g.setColor(Color.green);
		float x = 0, x2;
		float y;
		int xmax = getWidth();
		int ymax = getHeight();
		float dx = xmax/(float)power.length;
		for (int i = 0; i < power.length; i++){
			x2 = x + dx;
			// Compute bar height in pixels.
			y = (float)(
					// convert input power to dB
					10*Math.log(power[i]/(MAX_MEAN_SQUARE))/LOG10
					// scale to displayable dynamic range.
					* ymax/DYNAMIC_RANGE_dB
					// adjust zero position
					+ ymax
				);
			// Negative heights are zeroed.
			y = Math.max(0,y);
			// Draw bar i :
			// The "-1" leaves a black line between bars.
			g.fillRect((int)x, ymax-(int)y, (int)(x2-x)-1, (int)y); 
			x = x2;
		}
	}
}